<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco00500PadraoQualidade extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function salvaNovoPadraoQualidade(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $padraoQualidadeExistente = $this->retornaPadraoQualidadeExistente($row);
        
        if(empty($padraoQualidadeExistente)){
            if($this->isValid($row, $response)){
                $row["CDPADRAOQUALIDADE"] = str_pad($row["CDPADRAOQUALIDADE"], 5, "0", STR_PAD_LEFT);
                $this->salvaPadraoQualidade($request);
                $response->addDataSet(new DataSet("padraoQualidade", array()));
                $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
            }
        } else {
            $response->addDataSet(new DataSet("padraoQualidade", $padraoQualidadeExistente));
        }
    }
    
    private function isValid($row, $response) {
        $arrayErrors = array();
        
        if(strlen($row["CDPADRAOQUALIDADE"]) > 5){
            $response->addMessage(new Message('TAMANHO_CODIGO'));
            return false;
        }
        if(preg_match('/^([0-9]*)$/', $row["CDPADRAOQUALIDADE"]) < 1){
            $response->addMessage(new Message('CODIGO_INVALIDO'));
            return false;
        }
        if(strlen($row["NMPADRAOQUALIDADE"]) > 50){
            $response->addMessage(new Message('TAMANHO_NOME'));
            return false;
        }
        if(strlen($row["NMPADRAOQUALIDADE"]) == 0){
            $response->addMessage(new Message('NOME_VAZIO'));
            return false;
        }
        
        return true;
    }
    
    private function retornaPadraoQualidadeExistente($row) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT 'TCO00500_PADRAO_QUALIDADE_EXISTENTE' AS MESSAGECODE,
                       NRPADRAOQUALIDADE,
                       CDPADRAOQUALIDADE,
                       NMPADRAOQUALIDADE
                  FROM PADRAOQUALIDADE
                 WHERE CDPADRAOQUALIDADE = :CDPADRAOQUALIDADE
                   AND (NRPADRAOQUALIDADE <> :NRPADRAOQUALIDADE OR :NRPADRAOQUALIDADE IS NULL)
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDPADRAOQUALIDADE"  => str_pad($row["CDPADRAOQUALIDADE"], 5, "0", STR_PAD_LEFT),
            "NRPADRAOQUALIDADE"  => isset($row['NRPADRAOQUALIDADE']) ? $row['NRPADRAOQUALIDADE'] : null,
            "NRORG"              => $this->environment->getNrOrgTrab()
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function salvaPadraoQualidade($request) {
        $rows = $this->crudService->getRowsToSave($request, "padraoqualidade");
        $this->crudService->save("padraoqualidade", $rows);
        $rows[0]["__is_new"] = false;
        $this->crudService->addColumnToReturn($rows[0]);
    }
    
    public function removePadraoQualidadeSelecionado(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $padraoQualidadeEmUso = $this->retornaPadraoQualidadeEmUso($row);
        
        if(empty($padraoQualidadeEmUso)){
            $this->removePadraoQualidade($request, $response);
            $response->addDataSet(new DataSet("padraoQualidadeEmUso", array()));
        } else {
            $response->addDataSet(new DataSet("padraoQualidadeEmUso", $padraoQualidadeEmUso));
        }
        
    }
    
    private function retornaPadraoQualidadeEmUso($row) {
        $connection = $this->entityManager->getConnection();
        $sql = " SELECT MESSAGECODE,
                        PARAM1,
                        PARAM2,
                        PARAM3
                   FROM ( /*Padrão de qualidade em uso na receita por padrão de qualidade do prato __PARAM1__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_RECEPADQUALI' AS MESSAGECODE,                
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM1,
                                 NULL AS PARAM2,
                                 NULL AS PARAM3
                            FROM RECEPADQUALI R
                                 INNER JOIN PRATO P ON R.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       R.CDPRATO = P.CDPRATO                
                           WHERE R.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
                          
                           UNION ALL
                          /*Padrão de qualidade em uso nos dados da receita por padrão de qualidade do prato __PARAM1__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_PRATPADQUALI' AS MESSAGECODE,                
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM1,
                                 NULL AS PARAM2,
                                 NULL AS PARAM3
                            FROM PRATPADQUALI PQ
                                 INNER JOIN PRATO P ON PQ.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       PQ.CDPRATO = P.CDPRATO
                           WHERE PQ.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
              
                           UNION ALL
              
                          /*Padrão de qualidade em uso nos dados da receita da região __PARAM1__ do prato __PARAM2__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_PRATREGIAO' AS MESSAGECODE,                
                                 R.CDREGIAO || ' - ' || R.NMREGIAO AS PARAM1,
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM2,
                                 NULL AS PARAM3
                            FROM PRATREGIAO PR
                                 INNER JOIN PRATO P ON PR.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       PR.CDPRATO = P.CDPRATO
                                 INNER JOIN REGIAO R ON R.CDREGIAO = PR.CDREGIAO
                           WHERE PR.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
                           UNION ALL
              
                          /*Padrão de qualidade em uso na receita da região __PARAM1__ do prato __PARAM2__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_RECEREGIAO' AS MESSAGECODE,                
                                 RG.CDREGIAO || ' - ' || RG.NMREGIAO AS PARAM1,
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM2,
                                 NULL AS PARAM3
                            FROM RECEREGIAO R
                                 INNER JOIN PRATO P ON R.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       R.CDPRATO = P.CDPRATO
                                 INNER JOIN REGIAO RG ON RG.CDREGIAO = R.CDREGIAO
                           WHERE R.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
              
                           UNION ALL
              
                          /*Padrão de qualidade em uso no modo de preparo da região __PARAM1__ do prato __PARAM2__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_PREPPRAREGIAO' AS MESSAGECODE,                
                                 R.CDREGIAO || ' - ' || R.NMREGIAO AS PARAM1,
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM2,
                                 NULL AS PARAM3
                            FROM PREPPRAREGIAO PR
                                 INNER JOIN PRATO P ON PR.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       PR.CDPRATO = P.CDPRATO
                                 INNER JOIN REGIAO R ON R.CDREGIAO = PR.CDREGIAO
                           WHERE PR.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
                           UNION ALL
              
                          /*Padrão de qualidade em uso no modo de preparo por padrão de qualidade do prato __PARAM1__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_PREPPRAPADQUALI' AS MESSAGECODE,                
                                 P.CDPRATO || ' - ' || P.NMPRATO AS PARAM1,
                                 NULL AS PARAM2,
                                 NULL AS PARAM3
                            FROM PREPPRAPADQUALI PQ
                                 INNER JOIN PRATO P ON PQ.CDFILAUXPRAT = P.CDFILAUXPRAT AND
                                                       PQ.CDPRATO = P.CDPRATO
                           WHERE PQ.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
              
                           UNION ALL
                          /*Padrão de qualidade em uso no cardápio base __PARAM1__*/
                          SELECT 'TCO00500_PADQUALI_EM_USO_CARDAPIOBASE' AS MESSAGECODE,                
                                 C.CDCARDAPIOBASE AS PARAM1,
                                 NULL AS PARAM2,
                                 NULL AS PARAM3
                            FROM CARDAPIOBASE C
                           WHERE C.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
              
                          /*Padrão de qualidade em uso no serviço base __PARAM1__*/
                          UNION ALL
                          SELECT 'TCO00500_PADQUALI_EM_USO_SERVICOPROPCOM' AS MESSAGECODE,                
                                 SP.CDSERVICOPROPOSTA || ' - ' || SP.NMSERVICOPROPOSTA AS PARAM1,                
                                 V.CDVERSAOPROPCOM AS PARAM2,
                                 P.CDPROPOSTACOMERCIAL AS PARAM3
                            FROM SERVICOPROPCOM S
                                 INNER JOIN SERVICOPROPOSTA SP ON SP.NRSERVICOPROPOSTA = S.NRSERVICOPROPOSTA
                                 INNER JOIN VERSAOPROPCOM V ON V.NRVERSAOPROPCOM = S.NRVERSAOPROPCOM
                                 INNER JOIN PROPOSTACOMERCIAL P ON P.NRPROPOSTACOMERCIAL =  V.NRPROPOSTACOMERCIAL
                           WHERE S.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE
                       )";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRPADRAOQUALIDADE"  => $row['NRPADRAOQUALIDADE']
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function removePadraoQualidade($request, $response) {
        $rows = $this->crudService->getRowsToDelete($request, "padraoqualidade");
        $dataSet = new DataSet("padraoqualidade", $rows);
		$this->crudService->delete($response, $dataSet);
    }

}
